/**************************************************************
	LCD Serial communication monitor (inexpensive)
	8 columns 2 lines 
	 for PIC16F648A/628(A)
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.00
	Tools:
	MPLAB v8.50
	Hitech PIC C pro9.70
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20110615:0700	1.00	---		begin the program
---------------------------------------------------------------
	Pin assignment
	1:RA2:O:LCD DB6
	2:RA3:O:LCD DB7
	3:RA4:I:GND
	4:MCLR:I:PICKIT2-1(W/10kPU)
	5:VSS:GND
	6:RB0:I:MODE SWITCH
	7:RB1:I:Recive serial
	8:RB2:O:Transmit serial(Not used)
	9:RB3:O:PWM out(Use negative potential)

	18:RA1:O:LCD DB5
	17:RA0:O:LCD DB4
	16:CLOCK:8MHz Resonator1
	15:CLOCK:8MHz Resonator2
	14:VDD:Power 3 to 5 Volt
	13:RB7:O:LCD E
	12:RB6:O:LCD RS
	11:RB5:I:RIGHT SW
	10:RB4:I:LEFT SW

**************************************************************/
#include <htc.h>
#include "sci.h"
#include "lcd.h"

#define MHz 000000
#define _XTAL_FREQ 8MHz

#define	SW_MODE		RB0 //mode set
#define	SW_LEFT		RB4 //left key
#define	SW_RIGHT	RB5 //right key
#define DisplayMode	0x7f //eeprom display address*2.02
#define COM_Speed	0x7e //eeprom display address*2.02

//speed
#define Speed1200	250
#define Speed2400	251
#define Speed4800	252
#define Speed9600	253
#define Speed19200	254
#define Speed38400	255

#define MIN_Speed	250
#define MAX_Speed	255


#define ASCIImode	0xff
#define HEXmode		0xfe

//LCD location parameter
//ACM0802C-NLW-***(8 x 2 lines)
#define	LCD_LineNumber1	0x00
#define	LCD_LineNumber2	0x40
#define	LCD_CursorOn	0x0F
#define	LCD_CursorOff	0x0c




#define MAX_BUFF	64	//Ring buffer size 2^n

//__CONFIG(0x3F02);

__CONFIG(

	UNPROTECT & LVPDIS & BORDIS & MCLRDIS & PWRTEN & WDTDIS & HS

	);
//Global settings
//Fifo Data
unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char FIFO_write_pointer=0;
unsigned char FIFO_read_pointer=0;
unsigned char FIFO_datasize=0;

void interrupt handler( void )	//interrupt handle
{
	unsigned char temp;
	unsigned char data;
	//Handle RS232 Recieve interrupt
	if( RCIF == 1 ){
		//recive to data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 Recieve interrupt bit
		if(FIFO_datasize<MAX_BUFF){
			Buffer_FIFO[FIFO_write_pointer]=data;
			FIFO_write_pointer=(FIFO_write_pointer+1) & (MAX_BUFF-1);//ring buff 
			FIFO_datasize++;
		}
	}
	if( T0IF==1 ){	//timer interval interrupt
		if (!SW_LEFT){
			while(!SW_LEFT);// button release check
			asm("goto 0");//reset
		}
	
		if (!SW_RIGHT)	{
			temp = EEPROM_READ(DisplayMode);
			if (temp != ASCIImode ) EEPROM_WRITE(DisplayMode,ASCIImode);
				else EEPROM_WRITE(DisplayMode,HEXmode);
			while(!SW_RIGHT);// button release check
			asm("goto 0");//reset
		}

		if (sci_CheckOERR()) { //sci error
			CREN=1;
		}

	//timer interval
	T0IF =0 ;
	}
}

unsigned char Buffer_readdata(void){
	unsigned char wdat;
	//read FIFO buffer
	while(FIFO_datasize==0);	// pause if buffer is empty
	wdat=Buffer_FIFO[FIFO_read_pointer];
	FIFO_read_pointer=(FIFO_read_pointer+1)&(MAX_BUFF-1);//ring buff
	FIFO_datasize--;
	return wdat;//get wdat
//	return FIFO_datasize;//debug
}	

void Buffer_check(void){
	unsigned char data;
	if (FIFO_datasize==MAX_BUFF-1) {
		lcd_putch(0xFF);
		lcd_puts("Full");
		lcd_putch(0xFF);
		RCIE = 0;	//ʐM荞݋֎~
		FIFO_datasize 		=0;	//
		FIFO_write_pointer 	=0;
		FIFO_read_pointer 	=0;
		CREN =0;	//M~
		while (RCIF ==1){
			data = sci_GetByte();		//ǂ
		}
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		CREN = 1;	//MJn
		RCIE = 1;	//ʐM荞݋					
	}	
}

void delay_10ms(unsigned int temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}
	

//------------------main
main()
{
// Register
	unsigned int speed = 38400; //default communication speed set
	unsigned char temp,data,buffspeed;
//	Port initalize
//		  	  76543210
	PORTA = 0b11100000;	//Port Initalize
	PORTB = 0b00000000;	//Port Initalize
	TRISA = 0b11110000; //
	TRISB = 0b00110111; //
	CMCON = 0x07;		//PORTA is all digital I/O

//Init
	OPTION = 0b00000111; // prescale 1/256 RBPU= EN
	GIE  = 0;//Int off

//timer0 setting (interrupt event timer 3.328ms)
	T0IE = 0;//timer interrupt is off
	TMR0 = 0;//interrupt timer counter set

//PWM initialize setting (83.8KHz on RB3 @8MHz)
	CCP1CON = 0b00001100;// use PWM mode
	PR2 = 0x17; // resolution is 6.5bit mode
	CCPR1L = 0x0c; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
/*
//PWM test (31.25KHz on RB3 @8MHz) test only
	CCP1CON = 0b00001100;// use PWM mode
	PR2 = 0xff; // resolution is 8bit mode
	CCPR1L = 0x20; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
*/

//LCD init
//	DelayMs(200);	//stabilize time
	delay_10ms(20);
	lcd_init();		//lcd initalize
	lcd_clear();	//lcd clear
//setting mode
	if (!SW_MODE ){
		lcd_write(LCD_CursorOff);			//J[\\Ȃ
		lcd_goto(LCD_LineNumber2);
		lcd_puts("Set -  +");
		while(!SW_MODE);
		buffspeed = EEPROM_READ(COM_Speed);
		while(1){
			__delay_ms(50);		//DelayMs(50);
			lcd_goto(LCD_LineNumber1);
			switch (buffspeed){
				case Speed1200:
					lcd_puts(" 12");
					break;
				case Speed2400:
					lcd_puts(" 24");
					break;
				case Speed4800:
					lcd_puts(" 48");
					break;
				case Speed9600:
					lcd_puts(" 96");
					break;
				case Speed19200:
					lcd_puts("192");
					break;
				case Speed38400:
					lcd_puts("384");
					break;
				default: // data is broken
					buffspeed =Speed38400;
			}
			lcd_puts("00bps");
			
			//key check
			if (!SW_LEFT ) {
				if ( buffspeed > MIN_Speed) buffspeed--;
				while(!SW_LEFT);

			}
			if (!SW_RIGHT  ){
				if ( buffspeed < MAX_Speed) buffspeed++;
				while(!SW_RIGHT);

			}
			if (!SW_MODE  ){
				if (buffspeed != EEPROM_READ(COM_Speed)){
					EEPROM_WRITE(COM_Speed,buffspeed);
					lcd_goto(LCD_LineNumber2);
					lcd_puts("OK ");
					lcd_write(LCD_CursorOn);//J[\_
					//DelayMs(255);
					delay_10ms(50);	
				}
				while(!SW_MODE);
				break;
			}
		}
	}

	buffspeed = EEPROM_READ(COM_Speed);
//sci initialize
	switch (buffspeed){
		case Speed1200:
			speed = 1200;
			break;
		case Speed2400:
			speed = 2400;
			break;
		case Speed4800:
			speed = 4800;
			break;
		case Speed9600:
			speed = 9600;
			break;
		case Speed19200:
			speed = 19200;
			break;
		case Speed38400: //This speed is not accept HEX mode.
			speed = 38400;
			break;
		default:
			speed = 38400;
	}

	sci_Init(speed ,SCI_EIGHT);// Baud set and Bit set

//interrupt enable
//	INTEDG = 0; //RB0 Key interrupt fall edge
//	INTE = 1;//RB0 is interrupt mode
	T0IE = 1;//timer interrupt on
	RCIE = 1;//recive interrupt on
	PEIE = 1;//Peripheral interrupt Enable 
	GIE  = 1;//Global interrupt Enable 

//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN=1;
	}

//Main routiene
	lcd_init();
	lcd_clear();	//lcd clear
	temp = EEPROM_READ(DisplayMode);

	switch (temp){
		case HEXmode :
			//ASCII to HEX convert(Use less or equal 19200bps)
			lcd_puts("HEX:");
			lcd_goto(0x40);
			while(1)	{
				data= Buffer_readdata();//Buffer read 
			    //Send high nibble
			    if( (data>>4) > 9 )
			        lcd_putch( 'A' - 10 + (data>>4) );
			    	else lcd_putch( '0' + (data>>4) );
			    //Send low nibble
			    if( (data&0x0F) > 9 )
			        lcd_putch( 'A' - 10 + (data&0x0F) );
			    	else lcd_putch( '0' + (data&0x0F) );

				Buffer_check();	//Buffer check
			}
			break;	

		default:
			lcd_puts("ASCII:");
			lcd_goto(LCD_LineNumber2);
			while(1){
				data= Buffer_readdata();//Buffer read 
				switch ( data ){
					case 12	:	//clear
						lcd_clear();
						break;
					case 0x0b	:	//Verticcal tab
						lcd_newline(1);
						break;
					case 0x0d	:	//return command
						lcd_newline(0);
						break;
					case 0x11	:	//cursor off(ctrl +q)
						lcd_write(LCD_CursorOff);//J[\\Ȃ
						break;
					case 0x17	:	//cursor on(ctrl +w)
						lcd_write(LCD_CursorOn);//J[\\
						break;
					default		:
						if 	(( data >= 0x20 ) && ( data <= 0xff  )) lcd_putch(data);
						break;
				}
			}
		break;

	}

}
